#include "PreCompiledHeaders.hh"
#include "Lib/Script/CRTPLuaScriptObject.hh"

using namespace Vivy::Script;

int
FunctionDeclaration::TOSTRING(lua_State *const L) noexcept
{
    FunctionDeclaration *const self = CHECK(L, 1);
    std::string str(className);
    str += std::string_view(" { ");
    str += self->parentScript.empty() ? self->name : self->parentScript + "::" + self->name;
    str += std::string_view(" }");
    lua_pushstring(L, str.c_str());
    return 1;
}

int
FunctionDeclaration::setName(lua_State *const L) noexcept
{
    FunctionDeclaration::CHECK(L, 1)->name = CHECK_STRING_VIEW(L, 2);
    LUA_RETURN_NOTHING(L);
}

int
FunctionDeclaration::setImportFromScript(lua_State *const L) noexcept
{
    auto *const fun   = FunctionDeclaration::CHECK(L, 1);
    fun->parentScript = CHECK_STRING_VIEW(L, 2);
    fun->isImported   = true;
    LUA_RETURN_NOTHING(L);
}

int
FunctionDeclaration::pushToRuntime(lua_State *const L) noexcept
{
    FunctionDeclaration *const self = CHECK(L, 1);
    auto *const context             = LuaContext::getContext(L);

    lua_settop(L, 1);

    if (self->isImported)
        err(context) << "The function \"" << self->parentScript << "::" << self->name
                     << "\" is an imported function";

    else if (context->registerDeclaration(self) == LuaContext::Code::Error)
        context->failedWith("Failed to register function " + self->name);

    LUA_RETURN_NOTHING(L);
}

void
FunctionDeclaration::pushLuaFunction(lua_State *const L) const noexcept
{
    [[maybe_unused]] bool ok = PushFunctionFromRegistry(L, functionRegisterKey);
}

bool
FunctionDeclaration::isLocalToModule() const noexcept
{
    return parentScript.size() == 0;
}

void
FunctionDeclaration::getLuaFunctionFromStack(lua_State *const L) noexcept
{
    luaL_checktype(L, -1, LUA_TFUNCTION);
    functionRegisterKey = luaL_ref(L, LUA_REGISTRYINDEX);
}
