#include "PreCompiledHeaders.hh"
#include "Lib/Script/CRTPLuaScriptObject.hh"

using namespace Vivy::Script;

int
JobDeclaration::TOSTRING(lua_State *const L) noexcept
{
    JobDeclaration *const self = CHECK(L, 1);
    std::string str(className);
    str += std::string_view(" { ");

    str += self->parentScript.empty() ? self->name : self->parentScript + "::" + self->name;
    str += ", options: [";

    for (const OptionDeclaration *const opt : self->options) {
        for (auto const &[name, _] : opt->options)
            str += " " + name;
    }

    str += std::string_view(" ] }");
    lua_pushstring(L, str.c_str());
    return 1;
}

int
JobDeclaration::setName(lua_State *L) noexcept
{
    JobDeclaration::CHECK(L, 1)->name = CHECK_STRING_VIEW(L, 2);
    LUA_RETURN_NOTHING(L);
}

int
JobDeclaration::setOptions(lua_State *L) noexcept
{
    auto *const context       = LuaContext::getContext(L);
    JobDeclaration *const job = JobDeclaration::CHECK(L, 1);
    err(context) << "Set options for job '" << job->name << "'";

    IterateOverArray(L, 2, [&context, job, L]() noexcept -> bool {
        OptionDeclaration *opt = OptionDeclaration::CHECK(L, -1);

        for (const auto &[regName, value] : opt->options) {
            err(context) << "\tregister option \"" << regName << "\" with type \""
                         << value.getSignature() << "\"";
        }

        job->options.push_back(opt);
        job->addUserValue(L, 1); // Tie T[i] to job and pop stack
        return true;             // Value was popped from the stack
    });

    LUA_RETURN_NOTHING(L);
}

int
JobDeclaration::setImportFromScript(lua_State *L) noexcept
{
    auto *const job   = JobDeclaration::CHECK(L, 1);
    job->parentScript = CHECK_STRING_VIEW(L, 2);
    job->isImported   = true;
    LUA_RETURN_NOTHING(L);
}

int
JobDeclaration::pushToRuntime(lua_State *const L) noexcept
{
    JobDeclaration *const self = CHECK(L, 1);
    auto *const context        = LuaContext::getContext(L);

    lua_settop(L, 1);

    if (self->isImported)
        err(context)
            << "The job \"" << self->parentScript << "::" << self->name << "\" is an imported job";

    else if (context->registerDeclaration(self) == LuaContext::Code::Error)
        context->failedWith("Failed to register job " + self->name);

    LUA_RETURN_NOTHING(L);
}

bool
JobDeclaration::isLocalToModule() const noexcept
{
    return parentScript.size() == 0;
}

void
JobDeclaration::pushLuaFunction(lua_State *const L) const noexcept
{
    [[maybe_unused]] bool ok = PushFunctionFromRegistry(L, functionRegisterKey);
}

void
JobDeclaration::getLuaFunctionFromStack(lua_State *const L, const JobIteratorType itType) noexcept
{
    luaL_checktype(L, -1, LUA_TFUNCTION);
    functionRegisterKey = luaL_ref(L, LUA_REGISTRYINDEX);
    iterationType       = itType;
}
