#include "PreCompiledHeaders.hh"
#include "Lib/Script/ScriptDocument.hh"

#include <QFileInfo>
#include <QFile>
#include <QTextStream>

using namespace Vivy;

ScriptDocument::ScriptDocument(const QString &path)
    : AbstractDocument(AbstractDocument::Type::Script, path)
{
    const QFileInfo file(path);
    if (!file.exists())
        throw std::runtime_error("File don't exists, don't create ScriptDocument");
}

void
ScriptDocument::copy(const QString &newName)
{
    /* Compute new paths */
    const QFileInfo newPath(newName);
    copyWith(newPath, [=, this]() noexcept { name = newName; });
    emit documentChanged();
}

void
ScriptDocument::rename(const QString &newName)
{
    /* Compute new paths */
    const QFileInfo newPath(newName);
    renameWith(newPath, [=, this]() noexcept { name = newName; });
    emit documentChanged();
}

void
ScriptDocument::save()
{
    if (!textDocument)
        throw std::runtime_error("no text document attached to the ScriptDocument!");

    QFile file(name);
    if (file.open(QIODevice::WriteOnly, QIODevice::Text)) {
        throw std::runtime_error("Failed to open file in WO+Text mode: " + name.toStdString());
    }

    QTextStream writter(&file);
    writter << textDocument->toPlainText();
    file.close();
}

void
ScriptDocument::attachTextDocument(const QTextDocument *const txt) noexcept
{
    textDocument = txt;
}
