#include "TimingParams.hh"

using namespace Vivy;

TimingParams::TimingParams(QWidget *parent) noexcept
    : QWidget(parent)
{
    QVBoxLayout *vlayout = new QVBoxLayout;
    QHBoxLayout *hlayout = new QHBoxLayout;
    vlayout->addLayout(hlayout);

    zoomSlider = std::make_unique<QSlider>(new QSlider(Qt::Vertical));
    if (QSlider *slider = zoomSlider.get()) {
        slider->setMinimum(1);
        slider->setMaximum(100);
        slider->setSingleStep(0);
        slider->setPageStep(0);
        slider->setValue(0);
        // FIXME: set to false for now as it may be intensive while rebuilding the scene isn't optimized
        slider->setTracking(false);
        hlayout->addWidget(slider);
    }
    sensibilitySlider = std::make_unique<QSlider>(new QSlider(Qt::Vertical));
    if (QSlider *slider = sensibilitySlider.get()) {
        slider->setMinimum(0);
        slider->setMaximum(100);
        slider->setSingleStep(0);
        slider->setPageStep(0);
        slider->setValue(0);
        // FIXME: set to false for now as it may be intensive while rebuilding the scene isn't optimized
        slider->setTracking(false);
        hlayout->addWidget(slider);
    }

    rebuildSceneButton = std::make_unique<QPushButton>(new QPushButton(QString("Rebuild scene")));
    if (QPushButton *button = rebuildSceneButton.get()) {
        vlayout->addWidget(button);
    }

    setLayout(vlayout);
}

QSlider *
TimingParams::getZoomSlider() const noexcept
{
    return zoomSlider.get();
}

QSlider *
TimingParams::getSensibilitySlider() const noexcept
{
    return sensibilitySlider.get();
}

QPushButton *
TimingParams::getRebuildSceneButton() const noexcept
{
    return rebuildSceneButton.get();
}

void
TimingParams::adjustFlip(QRectF *rect, qreal left, qreal right) noexcept
{
    if (Q_UNLIKELY(right - rect->right() < 0)) {
        qreal w = rect->width();
        rect->adjust(-w, 0, -w, 0);
    }
}

void
TimingParams::adjustTranslate(QRectF *rect, qreal left, qreal right) noexcept
{
    qreal d;
    qreal hw = rect->width() / 2;
    if (Q_UNLIKELY((d = hw + left) > 0)) {
        rect->adjust(d, 0, d, 0);
    }
    if (Q_UNLIKELY((d = right - hw) < 0)) {
        rect->adjust(d, 0, d, 0);
    }
}

QString
TimingParams::printMajorTicks(int t) noexcept
{
    // TODO : adapt to different ticks (not everytime with "." first)
    QString ret(
        QStringLiteral("%1").arg(t % 60000 / 1000, t >= 60000 ? 2 : 1, 10, QLatin1Char('0')) +
        QString(".") + QString::number(t % 1000 / m_majorTicks));
    if (t >= 60000)
        ret.prepend(QString::number(t / 60000) + QString(":"));
    if (t >= 360000)
        ret.prepend(QString::number(t / 360000) + QString(":"));
    return ret;
}

QString
TimingParams::printCursor(int t) noexcept
{
    int absT = abs(t);
    // TODO : adapt to different ticks (not everytime with "." first)
    QString ret(
        QString::fromLatin1(t < 0 ? "-" : "" ) +
        QStringLiteral("%1").arg(absT % 60000 / 1000, absT >= 60000 ? 2 : 1, 10, QLatin1Char('0')) +
        QString(".") + QString::number(absT % 1000 / m_minorTicks));
    if (absT >= 60000)
        ret.prepend(QString::number(absT / 60000) + QString(":"));
    if (absT >= 360000)
        ret.prepend(QString::number(absT / 360000) + QString(":"));
    return ret;
}

void
TimingParams::updateRatios() noexcept
{
    m_wl = m_audioLength != 0 ? qreal(m_audioWidth) / qreal(m_audioLength) : 0;
    m_lw = m_audioWidth != 0 ? qreal(m_audioLength) / qreal(m_audioWidth) : 0;

    emit paramsChanged();
}
