#ifndef VIVY_SCRIPT_DOCUMENT_VIEW_H
#define VIVY_SCRIPT_DOCUMENT_VIEW_H

#ifndef __cplusplus
#error "This is a C++ header"
#endif

#include "PreCompiledHeaders.hh"
#include "VivyApplication.hh"
#include "Lib/Utils.hh"
#include "Lib/Log.hh"
#include "Lib/Script/ScriptDocument.hh"
#include "UI/AbstractDocumentView.hh"
#include "UI/FakeVim/FakeVimHandler.hh"
#include "UI/ScriptViews/EditorProxy.hh"
#include "UI/ScriptViews/ScriptHighlighter.hh"
#include "UI/ScriptViews/ScriptEditor.hh"

namespace Vivy
{
class ScriptDocumentView final : public AbstractDocumentView {
    Q_OBJECT
    VIVY_UNMOVABLE_OBJECT(ScriptDocumentView)
    VIVY_APP_LOGGABLE_OBJECT(ScriptDocumentView, logger)

public:
    explicit ScriptDocumentView(std::shared_ptr<ScriptDocument>, QWidget *parent = nullptr);
    ~ScriptDocumentView() override;

    void closeDocument() noexcept override;
    void openProperties() noexcept override;

    QString getDocumentTabName() const noexcept override;
    QString getDocumentTabToolTip() const noexcept override;
    QIcon getDocumentTabIcon() const noexcept override;
    AbstractDocument *getDocument() const noexcept override;

    void loadViews() noexcept override {}

    void setUseFakeVimEditor(bool yes) noexcept;

signals:
    void luaErrorFound(int, QString);

private:
    ScriptEditor *editor{ nullptr };
    EditorProxy *proxy{ nullptr };
    ScriptHighlighter *syntax{ nullptr };
    FakeVimHandler *handler{ nullptr };
    std::shared_ptr<ScriptDocument> document{ nullptr };
    QString lastLuaErrorMsg{};
    int lastLuaErrorLine{ -1 };
    bool isUsingFakeVim{ false };

    static void initHandler(FakeVimHandler *handler) noexcept;
    static void clearUndoRedo(QPlainTextEdit *editor) noexcept;
};
}

#endif // VIVY_PROPERTY_DOCUMENT_VIEW_H
