#pragma once

#include "PreCompiledHeaders.hh"
#include "VivyApplication.hh"
#include "Lib/Log.hh"
#include "UI/ScriptViews/ScriptEditor.hh"

namespace Vivy
{
class EditorProxy;
class FakeVimHandler;
struct ExCommand;

class EditorProxy final : public QObject {
    Q_OBJECT
    VIVY_UNMOVABLE_OBJECT(EditorProxy)
    VIVY_APP_LOGGABLE_OBJECT(EditorProxy, logger)

    explicit EditorProxy(QPlainTextEdit *widget) noexcept;

public:
    ~EditorProxy() override;

signals:
    void handleInput(const QString &keys);
    void requestSave();
    void requestSaveAndQuit();
    void requestQuit();
    void requestRun();

public slots:
    void changeStatusData(const QString &info) noexcept;
    void highlightMatches(const QString &pattern) noexcept;
    void changeStatusMessage(const QString &contents, int cursorPos) noexcept;
    void changeExtraInformation(const QString &info) noexcept;
    void updateStatusBar() noexcept;
    void handleExCommand(bool *handled, const ExCommand &cmd) noexcept;
    void requestSetBlockSelection(const QTextCursor &tc) noexcept;
    void requestDisableBlockSelection() noexcept;
    void updateBlockSelection() noexcept;
    void requestHasBlockSelection(bool *on) noexcept;
    void indentRegion(int beginBlock, int endBlock, QChar typedChar) noexcept;
    void checkForElectricCharacter(bool *result, QChar c) noexcept;

private:
    static int firstNonSpace(const QString &text) noexcept;

    void updateExtraSelections() noexcept;
    bool wantSaveAndQuit(const ExCommand &cmd) noexcept;
    bool wantSave(const ExCommand &cmd) noexcept;
    bool wantQuit(const ExCommand &cmd) noexcept;
    bool wantRun(const ExCommand &cmd) noexcept;

    QPlainTextEdit *widget;
    QString statusMessage;
    QString statusData;

    QList<QTextEdit::ExtraSelection> searchSelection;
    QList<QTextEdit::ExtraSelection> clearSelection;
    QList<QTextEdit::ExtraSelection> blockSelection;

public:
    static EditorProxy *connectSignals(FakeVimHandler *handler, QPlainTextEdit *editor) noexcept;
};

}
