#pragma once

#ifndef __cplusplus
#error "This is a C++ header"
#endif

#include "PreCompiledHeaders.hh"
#include "VivyApplication.hh"
#include "Lib/Log.hh"
#include "Lib/Utils.hh"

namespace Vivy
{
class ScriptEditor final : public QPlainTextEdit {
    Q_OBJECT
    VIVY_UNMOVABLE_OBJECT(ScriptEditor)
    VIVY_APP_LOGGABLE_OBJECT(ScriptEditor, logger)

    // Get the line numbers, private class
    class LineNumberArea final : public QWidget {
        VIVY_UNMOVABLE_OBJECT(LineNumberArea)
    public:
        LineNumberArea(ScriptEditor *editor) noexcept;

        QSize sizeHint() const noexcept override;

    protected:
        void paintEvent(QPaintEvent *event) noexcept override;

    private:
        ScriptEditor *scriptEditor{ nullptr };
    };

    // Number of spaces in a tab
    static constexpr int spacesPerTab        = 4;
    static inline const QString spacesForTab = QString(QStringLiteral(" ")).repeated(spacesPerTab);

public:
    ScriptEditor(QWidget *parent) noexcept;

    void lineNumberAreaPaintEvent(QPaintEvent *event) noexcept;
    int lineNumberAreaWidth() noexcept;

public slots:
    void updateLastLuaError(int, QString);

protected:
    void resizeEvent(QResizeEvent *) noexcept override;
    void keyPressEvent(QKeyEvent *) noexcept override;
    void paintEvent(QPaintEvent *) noexcept override;

private slots:
    void updateLineNumberAreaWidth(int newBlockCount) noexcept;
    void updateLineNumberArea(const QRect &rect, int dy) noexcept;

private:
    QWidget *lineNumberArea{ nullptr };
    QRect cursorRect{};
};
}
