#pragma once

#ifndef __cplusplus
#error "This is a C++ header"
#endif

#include "PreCompiledHeaders.hh"
#include "UI/Theme/Theme.hh"

namespace Vivy
{
class ScriptHighlighter final : public QSyntaxHighlighter {
    Q_OBJECT

    struct HighlightingRule {
        HighlightingRule(QRegularExpression pttrn, QTextCharFormat frmt);

        QRegularExpression pattern;
        QTextCharFormat format;
    };

    enum class BlockState { None = -1, Quote = 1, Comment = 2 };

public:
    ScriptHighlighter(QTextDocument *parent = nullptr) noexcept;

protected:
    void highlightBlock(const QString &text) noexcept override;

    void setCurrentBlockState(const BlockState) noexcept;
    BlockState previousBlockState() noexcept;

private:
    void highlightStringBlock(const QString &text, const BlockState) noexcept;
    void highlightCommentBlock(const QString &text, const BlockState) noexcept;

    void highlightBlock(const QString &text, const BlockState previous,
                        const BlockState toHighlight, const QRegularExpression &start, const QRegularExpression &end,
                        const QTextCharFormat &format) noexcept;

    void resetHighlightingRule() noexcept;

    std::vector<HighlightingRule> highlightingRules;

    QRegularExpression commentStartExpression;
    QRegularExpression commentEndExpression;
    QRegularExpression quoteStartExpression;
    QRegularExpression quoteEndExpression;

    QTextCharFormat keywordFormat;
    QTextCharFormat valueFormat;
    QTextCharFormat singleLineCommentFormat;
    QTextCharFormat quotationFormat;
    QTextCharFormat functionFormat;
    QTextCharFormat enumFormat;
    QTextCharFormat memberFormat;
};
}
